
import java.util.*;
import java.io.*;
import org.apache.tools.ant.*;

public class LoggerGeneratorTask extends Task {
    private File input;
    public void setInput(File input) {
        this.input = input;
    }
    
    private File output;
    public void setOutput(File output) {
        this.output = output;
    }
    
    public void execute() throws BuildException {
        try {
            if (input == null || !input.isFile())
                throw new BuildException("Invalid input: " + input);
            
            if (output == null)
                throw new BuildException("Invalid output: " + output);
            
            if (output.isFile() && input.lastModified() < output.lastModified())
                return;
            
            System.out.println("from " + input + " to " + output);
            new LoggerGenerator(output);
        } catch (Exception x) {
            if (x instanceof BuildException) {
                throw (BuildException) x;
            } else {
                throw new BuildException(x);
            }
        }
    }
}


class LoggerGenerator {
    private PrintWriter writer = null;
    public LoggerGenerator(File output) throws IOException {
        try {
            output.getParentFile().mkdirs();
            writer = new PrintWriter(new FileWriter(output));
            generate();
        } finally {
            if (writer != null)
                writer.close();
        }
    }
    
    
    private int indent = 0;
    private void println(String str) {
        str = str.trim();
        if (str.length() == 0) {
            return;
        }
        
        if (str.startsWith("}")) {
            indent--;
        }
        for (int i = 0; i < indent; i++) {
            writer.print("    ");
        }
        writer.println(str);
        
        if (str.endsWith("{")) {
            indent++;
        }
    }
    
    private void generate() {
        String[] lebelNames = { "LEVEL_FATAL", "LEVEL_ERROR", "LEVEL_WARNING", "LEVEL_INFORMATION", "LEVEL_VERBOSE", "LEVEL_VERBOSEST" };
        String[] methodNames = { "fatal", "error", "warn", "inform", "verbose", "verbosest" };
        
]]<<
        package jp.gr.java_conf.koto.notavacc;
        
        import java.util.*;
        import java.io.*;
        import java.text.*;
        import jp.gr.java_conf.koto.notavacc.parser.Original.*;
        
        public final class Logger {
            #for (int i = 0; i < lebelNames.length; i++) {
                public static final int >>[[ lebelNames[i] ]]<< = >>[[ i ]]<<;
            #}
            private final String[] informationTypeStrings = new String[] {
                "",
                "",
                "warning: ",
                "",
                "verbose information: ",
                "verboser information: ",
            };
            
            private final int verboseLevel;
            private final ResourceBundle boundle;
            private final PrintWriter writer;
            
            private boolean hasError = false;
            public boolean hasError() {
                return hasError;
            }
            
            public Logger() {
                this(0);
            }
            public Logger(int verboseLevel) {
                this(verboseLevel, null);
            }
            public Logger(int verboseLevel, ResourceBundle boundle) {
                this.verboseLevel = verboseLevel;
                this.boundle = boundle;
                writer = new PrintWriter(System.out);
                
                if (verboseLevel < LEVEL_VERBOSEST) {
                    informationTypeStrings[LEVEL_VERBOSE] = "";
                }
            }
            public void close() {
                writer.flush();
            }
            protected void finalize() {
                close();
            }
            
            public int verboseLevel() {
                return verboseLevel;
            }
            
            private String lineSeparator = System.getProperty("line.separator", "\n");
            private Map formatCache = new HashMap();
            public void report(int level, String sourceName, int line, String message, Object[] params) {
                if (verboseLevel >= level) {
                    boolean printed = false;
                    
                    // print the name of the source.
                    if (sourceName != null) {
                        if (sourceName.indexOf(':') < 0 && sourceName.indexOf(' ') < 0) {
                            writer.print(sourceName);
                        } else {
                            writer.print('"' + sourceName + '"');
                        }
                        writer.print(":");
                        printed = true;
                    }
                    
                    // print the line.
                    if (line > 0) {
                        writer.print(line);
                        writer.print(":");
                        printed = true;
                    }
                    
                    if (printed) {
                        writer.print(' ');
                    }
                    
                    // print the message type.
                    writer.print(informationTypeStrings[level]);
                    
                    // print the detail.
                    MessageFormat format = (MessageFormat) formatCache.get(message);
                    if (format == null) {
                        String s = message;
                        if (boundle != null) {
                            try {
                                s = boundle.getString(s);
                            } catch(MissingResourceException ignored) {
                            } catch(ClassCastException ignored) {
                            }
                        }
                        StringBuffer buffer = new StringBuffer();
                        for (int i = 0; i < s.length(); i++) {
                            char ch = s.charAt(i);
                            if (ch == '\n') {
                                buffer.append(lineSeparator);
                            } else {
                                buffer.append(ch);
                            }
                        }
                        format = new MessageFormat(buffer.toString());
                        formatCache.put(message, format);
                    }
                    writer.print(format.format(params));
                    
                    // the end of line.
                    writer.println();
                    writer.flush();
                }
                if (level <= LEVEL_ERROR) {
                    hasError = true;
                }
                if (level <= LEVEL_FATAL) {
                    fatal();
                }
            }
            
            public void fatal() {
                throw new FatalErrorException();
            }
            
            public static class FatalErrorException extends RuntimeException {
                private FatalErrorException() {
                    super("fatal error");
                }
            }
            
            private static final Object[] EMPTY_ARRAY = new Object[0];
            #for (int i = 0; i < methodNames.length; i++) {
                #for (int j = -1; j <= 4; j++) {
                    #StringBuffer params = new StringBuffer();
                    #StringBuffer args = new StringBuffer();
                    #if (j == -1) {
                        #params.append(", Object[] params");
                        #args.append("params");
                    #} else if (j == 0) {
                        #args.append("EMPTY_ARRAY");
                    #} else {
                        #args.append("new Object[] { ");
                        #for (int k = 1; k <= j; k++) {
                            #params.append(", Object p").append(k);
                            #args.append("p").append(k).append(", ");
                        #}
                        #args.append("}");
                    #}
                    #//public void >>[[ methodNames[i] ]]<<(String sourceName, int line, String message>>[[ params ]]<<) {
                    #//    report(>>[[ lebelNames[i] ]]<<, sourceName, line, message, >>[[ args ]]<<);
                    #//}
                    
                    public void >>[[ methodNames[i] ]]<<(String message>>[[ params ]]<<) {
                        report(>>[[ lebelNames[i] ]]<<, (String) null, -1, message, >>[[ args ]]<<);
                    }
                    public void >>[[ methodNames[i] ]]<<(File file, String message>>[[ params ]]<<) {
                        report(>>[[ lebelNames[i] ]]<<, Main.getFileName(file), -1, message, >>[[ args ]]<<);
                    }
                    public void >>[[ methodNames[i] ]]<<(File file, int line, String message>>[[ params ]]<<) {
                        report(>>[[ lebelNames[i] ]]<<, Main.getFileName(file), line, message, >>[[ args ]]<<);
                    }
                    public void >>[[ methodNames[i] ]]<<(String sourceName, int line, String message>>[[ params ]]<<) {
                        report(>>[[ lebelNames[i] ]]<<, sourceName, line, message, >>[[ args ]]<<);
                    }
                    public void >>[[ methodNames[i] ]]<<(Token token, String message>>[[ params ]]<<) {
                        String sourceName = null;
                        try {
                            sourceName = token.getSourceName();
                        } catch (UnsupportedOperationException x) {
                            // ignore.
                        }
                        int line = -1;
                        try {
                            line = token.getLine();
                        } catch (UnsupportedOperationException x) {
                            // ignore.
                        }
                        report(>>[[ lebelNames[i] ]]<<, sourceName, line, message, >>[[ args ]]<<);
                    }
                #}
                public void >>[[ methodNames[i] ]]<<(String message, Token token) {
                    >>[[ methodNames[i] ]]<<(token, message, token);
                }
                public void >>[[ methodNames[i] ]]<<(String message, Token token, Object param2) {
                    >>[[ methodNames[i] ]]<<(token, message, token, param2);
                }
                public void >>[[ methodNames[i] ]]<<(String message, Token token, Object param2, Object param3) {
                    >>[[ methodNames[i] ]]<<(token, message, token, param2, param3);
                }
                public void >>[[ methodNames[i] ]]<<(String message, Token token, Object param2, Object param3, Object param4) {
                    >>[[ methodNames[i] ]]<<(token, message, token, param2, param3, param4);
                }
            #}
        }
>>[[
    }
}

